@@ -20,8 +20,9 @@ use List::Util qw(shuffle);
 #     also adds a TEXT 'arg' column to file_to_queue for passing arguments
 # 14: modifies 'device' mb_total, mb_used to INT for devs > 16TB
 # 15: adds checksum table, adds 'hashtype' column to 'class' table
-# 16: adds 'readonly' state to enum in host table
-use constant SCHEMA_VERSION => 16;
+# 16: no-op, see 17
+# 17: adds 'readonly' state to enum in host table
+use constant SCHEMA_VERSION => 17;
 
 sub new {
     my ($class) = @_;
@@ -543,6 +544,7 @@ sub setup_database {
     $sto->upgrade_add_file_to_queue_arg;
     $sto->upgrade_modify_device_size;
     $sto->upgrade_add_class_hashtype;
+    $sto->upgrade_add_host_readonly;
 
     return 1;
 }
